#include "HTableView.h"
#include <QtGui/QHeaderView>
#include <QtGui/QScrollBar>
#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>
#include <QtGui/QX11Info>

HTableView::HTableView(QStyleOptionViewItem::Position  pos,
                       int                             minRows,
                       int                             maxRows,
                       QWidget*                        parent)
   : QTableView(parent), pos(pos), minRows(minRows), maxRows(maxRows)
{
}

QStyleOptionViewItem HTableView::viewOptions() const 
{
   QStyleOptionViewItem  opt = QTableView::viewOptions();
   opt.decorationPosition = pos;
   opt.showDecorationSelected = false;
   return opt;
}

int HTableView::frameOnlyAroundContentsPixels() const
{
   if (style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents, 
                          0, this)) {
      return style()->pixelMetric(QStyle::PM_DefaultFrameWidth) * 2;
   } else {
      return 0;
   }
}

QSize HTableView::sizeHint() const
{
   if (minRows != maxRows) {
      minRows = qMax(minRows, model()->rowCount());
      maxRows = minRows = qMin(minRows, maxRows);
   }

   // NOTE: The height returned from here is reduced by 23-28 pixels when
   //       the widget is first drawn.  I don't know why, but I suspect
   //       it believes the sizeHint is including the horizontal scrollbar
   //       and then is subtracting that out when it decides that it doesn't 
   //       need it.  I used to add 28 pixels (hard-coded) to workaround that.
   //       But it isn't needed any more in this tool because the widget is 
   //       resized after the application starts by resizeColumnsToContents 
   //       below.  And the sizeHint is honored when that happens.  Heaven
   //       knows why it's honored late but not early.

   int  lastColumn = model()->columnCount() - 1;
   int  w          = (frameWidth()
                      + (verticalHeader()->isHidden()
                         ? 0 : verticalHeader()->sizeHint().width())
                      + columnViewportPosition(lastColumn)
                      + columnWidth(lastColumn)
                      + frameOnlyAroundContentsPixels()
                      + verticalScrollBar()->sizeHint().width()
                      + frameWidth());
   int  h          = (frameWidth()
                      + (horizontalHeader()->isHidden()
                         ? 0 : horizontalHeader()->sizeHint().height())
                      + rowViewportPosition(0)
                      + (rowHeight(0) * minRows)
                      + frameWidth());

   return QSize(w, h);
}

void HTableView::sortByColumn(int column)
{
   if (model() == NULL) return;

   Qt::SortOrder  order;
   if (horizontalHeader()->sortIndicatorSection() == column) {
      if (horizontalHeader()->sortIndicatorOrder() == Qt::AscendingOrder) {
         order = Qt::DescendingOrder;
      } else {
         order = Qt::AscendingOrder;
      }
   } else {
      order = Qt::AscendingOrder;
   }
   horizontalHeader()->setSortIndicator(column, order);
   model()->sort(column, order);
}

void  HTableView::resizeColumnsToContents()
{
   QTableView::resizeColumnsToContents();

   QWidget*  widget = this;
   for (; widget != NULL; widget = widget->parentWidget()) {
      widget->updateGeometry();
   }

   QRect  screen  = QApplication::desktop()->screenGeometry(x11Info().screen());
   QSize  newSize = window()->sizeHint().boundedTo(QSize(screen.width()*9/10,
                                                         screen.height()*9/10))
                                        .expandedTo(window()->size());

   window()->resize(newSize);
}

int HTableView::sizeHintForColumn(int column) const
{
   if (model() == NULL) return -1;

   int                   result = 0;
   QStyleOptionViewItem  option = viewOptions();
   int                   tail   = model()->rowCount(rootIndex());
   int                   row;

   for (row = 0; row < tail; ++row) {
      QModelIndex  index = model()->index(row, column, rootIndex());
      result = qMax(result, itemDelegate()->sizeHint(option, index).width());
   }

   if (showGrid()) {
      result++;
   }

   return result;
}
